/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.io.File;
import java.util.Map;

import org.objectweb.fractal.adl.util.Printer;

/**
 * Linker wrapper.
 */
public class Linker extends AbstractBuilder implements LinkerBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the Linker interface
  // --------------------------------------------------------------------------

  public void link(final File destFile, final File[] objFiles,
      final String ldflags, final Map<Object, Object> context) throws Exception {
    final CommandLine cmd = new CommandLine();
    cmd.setExecutable(getContextArg("linker-command", context));

    // Creating attribute compiler arguments
    if (getFlags() != null) cmd.createArgument(getFlags());

    cmd.createArgument("-o " + destFile.getAbsolutePath());
    File libc = null;
    for (final File objFile : objFiles) {
      if (objFile.getName().endsWith("libc.a"))
        libc = objFile;
      else
        cmd.createArgument(objFile.getAbsolutePath());
    }
    if (libc != null) cmd.createArgument(libc.getAbsolutePath());

    final String attrLdPath = tryGetContextArg("ld-path", context);
    final String attrLdFlags = tryGetContextArg("ld-flags", context);
    final String attrLdScript = tryGetContextArg("linker-script", context);
    if (attrLdPath != null) cmd.createArgument(attrLdPath);
    if (attrLdFlags != null) cmd.createArgument(attrLdFlags);
    if (ldflags != null) cmd.createArgument(ldflags);
    if (attrLdScript != null) cmd.createArgument("-T " + attrLdScript);

    if (Printer.isDebugEnable())
      Printer.debug("[ld] " + cmd.getCommandLine());
    else if (Printer.isInfoEnable())
      Printer.info("[ld] " + destFile.getAbsolutePath());

    final Process child = Runtime.getRuntime().exec(cmd.getCommandLine());
    printCompilerMessages(child);
    if (child.waitFor() != 0)
      throw new CompilationException("Error while linking the file \""
          + destFile.getAbsolutePath() + "\"");
    child.destroy();
  }
}
