/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.controllers.binding;

import static org.objectweb.fractal.cecilia.adl.controllers.ControllerDecorationUtil.hasCustomBindingController;

import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.interfaces.InterfaceContainer;
import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.interfaces.ImportedInterfaceCompiler;
import org.objectweb.fractal.task.codegen.api.RequireSourceCodeProviderTask;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Adds a task to set up the binding controller of primitive components.
 */
public class PrimitiveBCCompiler extends ImportedInterfaceCompiler {

  @Override
  protected void createTask(final ComponentContainer container,
      final TaskMap tasks, final TypeInterface[] interfaces) {
    // if the component has a custom binding controller implementation, simply
    // return
    if (hasCustomBindingController((InterfaceContainer) container)) return;
    final TaskMap.TaskHole taskHole = createTask("binding-controller",
        interfaces, container, tasks);
    getTaskHole("controller", container, tasks).addDependency(taskHole,
        RequireSourceCodeProviderTask.SOURCE_CODE_PROVIDER_TASK_ROLE,
        "binding-controller");
  }
}
