/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.implementations;

import java.util.Map;

import org.objectweb.fractal.task.codegen.util.CodeWriter;
import org.objectweb.fractal.task.codegen.util.SourceFile;

/**
 * Builds the definition of the implementation of a component written in
 * <code>thinkMC</code> language.
 */
public class ModuleImplementationBuilder
    implements
      org.objectweb.fractal.cecilia.adl.implementations.ImplementationBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the ImplementationBuilder interface
  // --------------------------------------------------------------------------

  public Object defineImplementation(final Object componentType,
      final Object contentDesc, final Object controllerDesc,
      final Map<String, Object> codePieces, final Map<Object, Object> context) {
    final CodeWriter cw = new CodeWriter("Implementation Definition Builder");
    final String contentPath = ((SourceFile) contentDesc).getFile()
        .getAbsolutePath();
    final String componentCName = componentType.toString().replace('.', '_');

    /* Define specified DECLARE_DATA macro for this component */
    cw.append("#define DECLARE_DATA \\").endl();
    cw.append("  struct ").append(componentCName).append("_instancedata")
        .endl();
    /* Define the specific METHOD macro for this component */
    cw.append("#define METHOD(i, m) \\").endl();
    cw.append(componentCName).append("##_##i##_##m##_method").endl();
    cw.append("#define CALLMINE(itf,proc,args...) \\").endl();
    cw.append("  ").append(componentCName).append(
        "##_##itf##_##proc##_method(_this, ##args)").endl();
    /* Define REQUIRED, ATTRIBUTES and DATA as a component specific macros */
    cw.append("#define REQUIRED \\").endl();
    cw.append("  ((struct ").append(componentCName).append(
        "_t *)_this)->type.imported ").endl();
    cw.append("#define ATTRIBUTES \\").endl();
    cw.append("  ((struct ").append(componentCName).append(
        "_t *)_this)->attributes ").endl();
    cw.append("#define DATA \\").endl();
    cw.append("  ((struct ").append(componentCName + "_t *)_this)->data ")
        .endl();
    /* Define the GETSELF macro according to above defined structures */
    cw.append("#define GETSELF \\").endl();
    cw.append("  struct ").append(componentCName).append(
        "_t *self = (struct " + componentCName).append("_t *) _this ").endl();

    // Define the GET_MY_OWNER macro
    cw.append("#define GET_MY_OWNER \\").endl();
    cw.append("  ((Rfractal_api_Component *) _this)").endl();

    // Define the GET_MY_INTERFACE macro
    cw.append("#define GET_MY_INTERFACE(itf) \\").endl();
    cw.append("  (&(((struct ").append(componentCName).append(
        "_t *)GET_MY_OWNER)->type.exported.itf))").endl();

    // Print the include lines
    cw.append("#define NEWSTYLE_COMPONENT").endl();
    cw.append("#include <").append(contentPath).append(">").endl();
    cw.append("#undef NEWSTYLE_COMPONENT").endl();

    return cw.toString();
  }

}
