/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 *
 */


package org.objectweb.fractal.cecilia.adl.idl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Map;

import org.objectweb.fractal.cecilia.adl.idl.ast.Field;
import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinition;
import org.objectweb.fractal.cecilia.adl.idl.ast.Import;
import org.objectweb.fractal.cecilia.adl.idl.ast.ImportContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.InterfaceDefinition;
import org.objectweb.fractal.cecilia.adl.idl.ast.Method;
import org.objectweb.fractal.cecilia.adl.idl.ast.PrimitiveType;
import org.objectweb.fractal.cecilia.adl.idl.ast.TypeContainer;
import org.objectweb.fractal.cecilia.adl.idl.util.CUtil;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds interface definitions for C. It extends the Abstract Interface visitor
 * and implements the required visit methods.
 */
public class CInterfaceBuilder extends AbstractInterfaceVisitor
    implements
      IDLBuilder {
  protected String     itfName;
  protected String     cItfName;
  protected CodeWriter cw = null;

  public void enterInterface(final InterfaceDefinition itf) throws Exception {
    cw.append("/* Interface ").append(itfName).endl();
    cw.appendln("Generated automatically, do not edit ! */");
    cw.endl();

    cw.append("#ifndef H").append(cItfName).endl();
    cw.append("#define H").append(cItfName).endl();
    cw.endl();

    cw.append("// Interface descriptor");
    cw.append("struct M").append(cItfName).append(";").endl();
    cw.appendln("typedef struct {");
    cw.append("  struct M").append(cItfName).append(" *meth;").endl();
    cw.appendln("  void *selfdata;");
    cw.appendln("#if defined(TYPEDINTERFACE)");
    cw.appendln("  char *type;");
    cw.appendln("#endif");
    cw.append("} R").append(cItfName).append(";").endl();

    // Add the includes for complex types
    if (itf instanceof ImportContainer) {
      for (final Import imp : ((ImportContainer) itf).getImports()) {
        cw.append(CUtil.buildInclude(imp.getName()));
      }
    }

    cw.append("#ifndef HTYPES_JAVA").endl();
    cw.append("#define HTYPES_JAVA").endl();
    cw.append("  typedef unsigned char    jboolean;").endl();
    cw.append("  typedef unsigned short   jchar;").endl();
    cw.append("  typedef signed char      jbyte;").endl();
    cw.append("  typedef signed short     jshort;").endl();
    cw.append("  typedef signed int       jint;").endl();
    cw.append("  typedef signed long long jlong;").endl();
    cw.append("  typedef unsigned char      jubyte;").endl();
    cw.append("  typedef unsigned short     jushort;").endl();
    cw.append("  typedef unsigned int       juint;").endl();
    cw.append("  typedef unsigned long long julong;").endl();
    cw.append("  typedef float            jfloat;").endl();
    cw.append("  typedef double           jdouble;").endl();
    cw.append("#endif /* HTYPES_JAVA */").endl();
    cw.endl();

    /*
     * cw.append("#if !defined(STOPIMPORT)"); cw.append("#if defined(HR" +
     * _smallinterface + ")"); cw.append("# error reference to " +
     * _smallinterface + " is ambiguous, both includes match");
     * cw.append("#endif"); cw.append("typedef R" + _interface.replace('.', '_') + "
     * R" + _smallinterface +";"); cw.append("#define HR" + _smallinterface);
     * cw.append("#endif");
     */
    cw.endl();
    cw.appendln("  /* Virtual table */");
    cw.append("struct M").append(cItfName).append(" {").endl();
  }

  public void leaveInterface(final InterfaceDefinition itf) throws Exception {
    cw.append("};").endl().endl();
    cw.appendln("#endif");
  }

  public void visitMethod(final Method method) throws Exception {
    cw.append(CUtil.buildMethodDefinition(method, true)).append(";").endl();
  }

  public void visitField(final Field field) throws Exception {
    final PrimitiveType type = ((TypeContainer) field).getPrimitiveType();
    cw.append("#define ").append(itfName.replace('.', '_')).append("_").append(
        field.getName()).append(" ((").append(CUtil.buildPrimitiveType(type))
        .append(") ").append(field.getValue()).append(')').endl();
    cw.endl();
  }

  public Object createIDLDefinition(final Object componentType,
      final Object componentInstance, final IDLDefinition idlDefinition,
      final Map<String, Object> codePieces, final Map<Object, Object> context)
      throws Exception {
    cw = new CodeWriter("C Interface builder");
    // Pre-visit operations
    itfName = idlDefinition.getName();
    cItfName = itfName.replace('.', '_');
    // Visit
    visit(idlDefinition);
    // Post-visit operations
    final File file = new File(context.get("idlBuildDirectory").toString(),
        idlDefinition.getName().replace('.', File.separatorChar) + ".idl.h");
    if (!file.getParentFile().exists()) file.getParentFile().mkdirs();

    final PrintStream out = new PrintStream(new FileOutputStream(file));
    out.print(cw.toString());
    out.close();

    return file;
  }
}
