/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.source;

import java.util.Collection;
import java.util.Map;

import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.cecilia.adl.compiler.AbstractOutputCompiler;
import org.objectweb.fractal.task.codegen.api.SourceFileProviderTask;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Interface provided by the SourceFileCompiler and used by the
 * {@link AbstractOutputCompiler OutputCompiler} to find the set of file to be
 * compiled.
 */
public interface SourceFileProvider {

  /**
   * Returns the collection of the {@link TaskMap.TaskHole tasks} that provide
   * source file to be compiled by the
   * {@link AbstractOutputCompiler OutputCompiler}. These task must be
   * {@link SourceFileProviderTask}.
   * 
   * @param container the component for which sources has to be compiled
   * @param tasks the task map.
   * @param context optional additional information.
   * @return a collection of the {@link TaskMap.TaskHole}. These task hole must
   *         be filled with {@link SourceFileProviderTask}.
   */
  Collection<TaskMap.TaskHole> getSourceFileProviderTask(
      ComponentContainer container, TaskMap tasks, Map<Object, Object> context);
}
