/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.types;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * This component adds a
 * {@link org.objectweb.fractal.task.codegen.api.FactoryProviderTask "type" task} and a
 * {@link org.objectweb.fractal.task.codegen.api.InstanceProviderTask "instance" task} and
 * call the {@link #definitionCompilerItf definition compiler} client interface.
 */
public class CompositeDefinitionCompiler extends PrimitiveDefinitionCompiler {

  @Override
  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {

    final String fullName = getDefinitionName(path, container);
    final String instanceName = getInstanceName(path, container);

    // set the type decoration
    TypeDecorationUtil.setTypeDecoration(container, container);

    // add the type name provider task
    tasks.addTask("type", container, new TypeTask(fullName));

    // call the client definition compiler for this new component type
    definitionCompilerItf.compile(path, container, tasks, context);

    // add the instance name provider task
    tasks.addTask("instance", container, new InstanceTask(instanceName));
  }
}
