/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.types;

import static org.objectweb.fractal.cecilia.adl.controllers.ControllerDecorationUtil.getVTable;

import java.io.File;
import java.util.Map;

import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the declaration/initialization of the VFT of a component written in
 * <code>thinkMC</code> language. The VFT declared by this builder is supposed
 * to be shared by each instance of the same component type. The code generated
 * by this builder is surrounded by pre-processing instructions so that it can be
 * used multiple time in the same files without causing multiple declaration.
 */
public class ExportedVFTDeclarationBuilder implements InterfaceBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the InterfaceBuilder interface
  // --------------------------------------------------------------------------

  public Object createInterfaces(final Object componentType,
      final Object componentInstance, final Object contentType,
      final TypeInterface[] interfaceTypes,
      final Map<String, Object> codePieces, final Map<Object, Object> context) {
    final String name = componentType.toString().replace('.', '_');
    final CodeWriter cw = new CodeWriter(
        "Exported Interface Definition Builder, VFT declaration");

    cw.append("#ifndef ").append(name).append("_VFT_DECLARATION").endl();
    cw.append("#define ").append(name).append("_VFT_DECLARATION").endl();
    cw.endl();

    for (final TypeInterface itf : interfaceTypes) {
      if (getVTable(itf) != null) continue;

      // include VFTTemplate header file
      cw.append("#include \"").append(
          itf.getSignature().replace('.', File.separatorChar)).append(
          "VFTTemplate.h\"").endl();
    }

    for (final TypeInterface itf : interfaceTypes) {
      if (getVTable(itf) != null) continue;

      // use the <itf>_VFT_DECLARATION macro to declare and initialize the VFT.
      cw.append("// declare and initialize the VFT for the interface").append(
          itf.getName()).endl();
      cw.append(itf.getSignature().replace('.', '_').toUpperCase()).append(
          "_VFT_DECLARATION(").append(name).append(",").append(
          itf.getName().replace('-', '_')).append(") ;").endl();
    }

    cw.append("#endif //").append(name).append("_VFT_DECLARATION").endl();

    return cw.toString();
  }

}
