/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.types;

import java.util.Map;

import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the definition of the interfaces of a component written in
 * <code>thinkMC</code> language.
 */
public class InterfaceDefinitionBuilder implements InterfaceBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the InterfaceBuilder interface
  // --------------------------------------------------------------------------

  public Object createInterfaces(final Object componentType,
      final Object componentInstance, final Object contentType,
      final TypeInterface[] interfaceTypes,
      final Map<String, Object> codePieces, final Map<Object, Object> context) {

    // generate the definition of the structure representing the component type
    final CodeWriter cw = new CodeWriter("Interface Definition Builder, type");
    final String name = componentType.toString().replace('.', '_');

    final String exporteds = (String) codePieces.get("exported-interfaces");
    final String importeds = (String) codePieces.get("imported-interfaces");

    // This code is supposed to defines the structure <name>_exporteds
    cw.append(exporteds);

    // This code is supposed to defines the structure <name>_importeds
    cw.append(importeds);

    // defines the structure <name>_type
    cw.append("struct ").append(name).append("_type {").endl();
    if (exporteds != null)
      cw.append("struct ").append(name).append("_exporteds exported;").endl();
    if (importeds != null)
      cw.append("struct ").append(name).append("_importeds imported;").endl();
    cw.append("};").endl();

    return cw.toString();
  }
}
